@@ -22,9 +22,12 @@
 #  include <config.h>
 #endif
 
+#include <stdlib.h>
+#include <string.h>
+
 #include "canvas.h"
 #include "callbacks.h"
-#include "support.h"
+#include "util.h"
 #include "debug.h"
 #include <gtk/gtk.h>
 
@@ -197,7 +200,8 @@ on_color_palette_entry_expose_event    (GtkWidget       *widget,
     gpaint_color_swatch *swatch = 
         (gpaint_color_swatch*)gtk_object_get_data(GTK_OBJECT(widget), "color_swatch");
     debug_fn1("widget=%s", gtk_widget_get_name(widget));
-    g_assert(swatch);
+    if (swatch == NULL)
+        return FALSE;
     gdk_draw_rectangle(
             widget->window, swatch->gc, TRUE,
             event->area.x, event->area.y,
@@ -334,7 +338,7 @@ change_color(gpaint_color_swatch *swatch, gpaint_color_mode mode)
     g_assert(swatch);
     canvas = canvas_lookup(swatch->widget);
     gdk_gc_get_values(swatch->gc, &gcvalues);
-    
+
     if (mode==FOREGROUND)
     {
         change_foreground_color(canvas, &(gcvalues.foreground));
@@ -358,15 +362,12 @@ change_color(gpaint_color_swatch *swatch, gpaint_color_mode mode)
  */
 void
 on_foreground_color_picker_color_set   (GtkColorButton *gnomecolorpicker,
-                                        guint            arg1,
-                                        guint            arg2,
-                                        guint            arg3,
-                                        guint            arg4,
                                         gpointer         user_data)
 {
-    GdkColor color = {0, arg1, arg2, arg3};
+    GdkColor color;
     gpaint_canvas *canvas = canvas_lookup(GTK_WIDGET(gnomecolorpicker));
     
+    gtk_color_button_get_color(gnomecolorpicker, &color);
     gdk_color_alloc(gdk_colormap_get_system(), &color);
     change_foreground_color(canvas, &color);
 }
@@ -376,15 +377,12 @@ on_foreground_color_picker_color_set   (GtkColorButton *gnomecolorpicker,
  */
 void
 on_background_color_picker_color_set   (GtkColorButton *gnomecolorpicker,
-                                        guint            arg1,
-                                        guint            arg2,
-                                        guint            arg3,
-                                        guint            arg4,
                                         gpointer         user_data)
 {
-    GdkColor color = {0, arg1, arg2, arg3};
+    GdkColor color;
     gpaint_canvas *canvas = canvas_lookup(GTK_WIDGET(gnomecolorpicker));
 
+    gtk_color_button_get_color(gnomecolorpicker, &color);
     gdk_color_alloc(gdk_colormap_get_system(), &color);
     change_background_color(canvas, &color);
 }
