@@ -1,11 +1,11 @@
 /* $Id: image.c,v 1.7 2005/02/01 02:42:50 meffie Exp $
  *
- * GNU Paint 
+ * GNU Paint
  * Copyright 2000-2003, 2007  Li-Cheng (Andy) Tai
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
+ * as published by the Free Software Foundation; either version 3
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be
@@ -13,10 +13,8 @@
  * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  * PURPOSE. See the GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,19 +27,23 @@
 
 #include "image.h"
 #include "debug.h"
-#include <math.h>     
+#include <math.h>
+#include <string.h>
+#include <libintl.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkx.h>  /* for gdk_root_parent */
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+#define _(String) gettext (String)
 
 static int cmp_int(const void *a, const void *b);
 static void fill_polygon(
-        unsigned char *data, 
+        unsigned char *data,
         int pixelsize,
-        int rowstride, 
-        unsigned char value[4], 
-        unsigned char mask[4], 
-        GdkPoint *pts, 
+        int rowstride,
+        unsigned char value[4],
+        unsigned char mask[4],
+        GdkPoint *pts,
         int num_pts,
         int xoffset,
         int yoffset);
@@ -50,9 +52,19 @@ const int BITS_PER_SAMPLE = 8;
 
 struct _gpaint_image
 {
-    GdkPixbuf *pixbuf;    /* X client side image for processing */ 
+    GdkPixbuf *pixbuf;    /* X client side image for processing */
 };
 
+
+typedef struct {
+  int rowstride;
+  int pixelsize;
+  unsigned char value[4];
+  unsigned char mask[4];
+  unsigned char * data;
+} GdkSpanFillData;
+
+
 gpaint_image*
 image_new(int width, int height)
 {
@@ -74,16 +86,16 @@ image_new_copy(const gpaint_image *source)
     return image;
 }
 
-gpaint_image* 
+gpaint_image*
 image_new_from_pixmap(GdkPixmap* pixmap, GdkRectangle source)
 {
     gpaint_image *image = image_new(source.width, source.height);
     gdk_pixbuf_get_from_drawable(
                 image->pixbuf,
                 pixmap,
-                gdk_rgb_get_cmap(), 
+                gdk_rgb_get_cmap(),
                 source.x, source.y,
-                0, 0, /* destination x, y */ 
+                0, 0, /* destination x, y */
                 source.width, source.height);
     g_assert(image->pixbuf);
     return image;
@@ -92,10 +104,10 @@ image_new_from_pixmap(GdkPixmap* pixmap, GdkRectangle source)
 gpaint_image*
 image_new_from_desktop()
 {
-    gpaint_image *image; 
+    gpaint_image *image;
     GdkWindow *root = gdk_window_lookup(gdk_x11_get_default_root_xwindow());
     int width, height;
-    
+
     gdk_window_get_size(root, &width, &height);
     image = image_new(width, height);
     gdk_pixbuf_get_from_drawable(image->pixbuf, root, 0, 0, 0, 0, 0, width, height);
@@ -106,7 +118,7 @@ image_new_from_desktop()
 void
 image_free(gpaint_image* image)
 {
-    g_assert(image);    
+    g_assert(image);
     gdk_pixbuf_unref(image->pixbuf);
     g_free(image);
 }
@@ -120,7 +132,7 @@ image_draw_on_pixmap(gpaint_image* image, GdkPixmap** ppixmap, GdkGC *gc)
     g_assert(image && image->pixbuf);
     g_assert(ppixmap && &ppixmap);
     g_assert(gc);
-    
+
     width = image_width(image);
     height = image_height(image);
     gdk_pixbuf_render_to_drawable(
@@ -131,7 +143,7 @@ image_draw_on_pixmap(gpaint_image* image, GdkPixmap** ppixmap, GdkGC *gc)
             0, 0, width, height,
             GDK_RGB_DITHER_NORMAL, 0, 0);
 
-    return 0; 
+    return 0;
 }
 
 int
@@ -151,13 +163,13 @@ image_draw_region_on_pixmap(
             rect.x, rect.y,
             rect.x, rect.y, rect.width, rect.height,
             GDK_RGB_DITHER_NORMAL, 0, 0);
-    return 0; 
+    return 0;
 }
 
 int
 image_render(gpaint_image *image, GdkPixmap *pixmap, gint x, gint y, GdkRectangle rect)
 {
-    gdk_pixbuf_render_to_drawable_alpha( 
+    gdk_pixbuf_render_to_drawable_alpha(
            image->pixbuf, pixmap,
            x, y,
            rect.x, rect.y, rect.width, rect.height,
@@ -172,7 +184,7 @@ image_new_readfile(const gchar* filename)
 {
     GdkPixbuf *pixbuf = NULL;
     gpaint_image *image = NULL;
-    
+
     pixbuf = gdk_pixbuf_new_from_file(filename,0);
     if (pixbuf)
     {
@@ -182,13 +194,48 @@ image_new_readfile(const gchar* filename)
     return image;
 }
 
+
+gboolean
+is_writable (GSList *formats, gchar *type)
+{
+    gboolean writable = FALSE;
+
+    while (!writable && formats != NULL)
+    {
+        gchar** extensions = gdk_pixbuf_format_get_extensions((GdkPixbufFormat *)(formats->data));
+        gchar** e = NULL;
+        for (e = extensions; *e; ++e)
+        {
+            if (!strcmp(*e, type))
+            {
+                writable = TRUE;
+                break;
+            }
+        }
+        g_strfreev (extensions);
+        formats = g_slist_next (formats);
+    }
+    return writable;
+}
+
+void add_if_writable (GdkPixbufFormat *data, GSList **list)
+{
+  if (gdk_pixbuf_format_is_writable (data))
+    *list = g_slist_prepend (*list, data);
+}
+
 int
 image_write(gpaint_image* image, const gchar* filename, GError **perror)
 {
     gboolean saved = FALSE;
     gchar *ext = NULL;
     gchar *type = NULL;
-    
+
+    GSList *formats = gdk_pixbuf_get_formats ();
+    GSList *writable_formats = NULL;
+    g_slist_foreach (formats, add_if_writable, &writable_formats);
+    g_slist_free (formats);
+
     ext = g_strrstr(filename, ".");
     if (!ext)
     {
@@ -205,20 +252,27 @@ image_write(gpaint_image* image, const gchar* filename, GError **perror)
     {
         type = g_ascii_strdown(ext+1,-1);
         debug1("type=[%s]",type);
-        saved = gdk_pixbuf_save(image->pixbuf, filename, type, perror, NULL);
+        if (is_writable (writable_formats, type))
+        {
+            saved = gdk_pixbuf_save(image->pixbuf, filename, type, perror, NULL);
+        } else {
+            saved = FALSE;
+            *perror = g_error_new (GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_BAD_OPTION, _("Saving in the '%s' format is not supported by gdk_pixbuf"), type);
+        }
         g_free(type);
     }
     if (!saved && *perror)
     {
         g_warning("Could not save image %s: %s\n", filename, (*perror)->message);
-    }      
+    }
+	g_slist_free (writable_formats);
     return saved;
 }
 
 int
 image_print(gpaint_image* image)
 {
-    return 0; 
+    return 0;
 }
 
 int
@@ -248,7 +302,7 @@ image_rowstride(gpaint_image *image)
 int
 image_pixelsize(gpaint_image *image)
 {
-   return gdk_pixbuf_get_n_channels(image->pixbuf) 
+   return gdk_pixbuf_get_n_channels(image->pixbuf)
           * gdk_pixbuf_get_bits_per_sample(image->pixbuf) / BITS_PER_SAMPLE;
 }
 
@@ -264,14 +318,14 @@ image_from_selection(GdkPixmap *pixmap, gpaint_point_array *pa)
     unsigned char alphaonly[4] = {0, 0, 0, 1};
     unsigned char *data = 0, *c, *d;
     unsigned char default_pixel[4] = {0, 0, 0, 255};
-    int width, height; 
+    int width, height;
     int x, y;
     int pixel_size;
     GdkPixbuf *temp_pixbuf;
     GdkPoint *pts = point_array_data(pa);
     int num_pts   = point_array_size(pa);
     const int BITS_PER_SAMPLE = 8;
-   
+
     point_array_bounding_rectangle(pa, &sel_rect);
     gdk_window_get_size((GdkWindow*)pixmap, &width, &height);
     pixmap_rect.x = 0;
@@ -288,20 +342,20 @@ image_from_selection(GdkPixmap *pixmap, gpaint_point_array *pa)
     debug1("rect.y=%d", rect.y);
     debug1("rect.width=%d", rect.width);
     debug1("rect.height=%d", rect.height);
-    
-    temp_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, BITS_PER_SAMPLE, rect.width, rect.height);   
-    gdk_pixbuf_get_from_drawable(temp_pixbuf, 
-           pixmap, 
-           gdk_rgb_get_cmap(), 
+
+    temp_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, BITS_PER_SAMPLE, rect.width, rect.height);
+    gdk_pixbuf_get_from_drawable(temp_pixbuf,
+           pixmap,
+           gdk_rgb_get_cmap(),
            rect.x, rect.y, 0, 0, rect.width, rect.height);
 
     pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, rect.width, rect.height);
     g_assert(pixbuf);
     pixel_size = gdk_pixbuf_get_n_channels(pixbuf) * gdk_pixbuf_get_bits_per_sample(pixbuf) / 8;
-   
+
     gdk_pixbuf_copy_area(temp_pixbuf, 0, 0, rect.width, rect.height, pixbuf, 0, 0);
     gdk_pixbuf_unref(temp_pixbuf);
-   
+
     data = gdk_pixbuf_get_pixels(pixbuf);
     for (y = 0, c = data; y < rect.height; y++, c += gdk_pixbuf_get_rowstride(pixbuf))
     {
@@ -311,14 +365,14 @@ image_from_selection(GdkPixmap *pixmap, gpaint_point_array *pa)
             *(d + 3) = 0;/* default everything is transparent */
         }
     }
-   
-    fill_polygon(data, 
+
+    fill_polygon(data,
                 pixel_size,
                 gdk_pixbuf_get_rowstride(pixbuf),
                 default_pixel,
                 alphaonly,
                 pts,
-                num_pts, 
+                num_pts,
                 rect.x, rect.y);
 
     debug("fill_polygon() done");
@@ -333,113 +387,78 @@ cmp_int(const void *a, const void *b)
 	return (*(const int *)a) - (*(const int *)b);
 }
 
+
+
+static void gdk_span_fill(GdkSpan *span, gpointer data) {
+  GdkSpanFillData *fill_data = (GdkSpanFillData *) data;
+  unsigned char *c = fill_data->data + (fill_data->rowstride * span->y) + (fill_data->pixelsize * span->x);
+  int i;
+  int j;
+
+  for (i = 0; i <span->width; ++i) {
+    for (j = 0; j < fill_data->pixelsize; ++j, ++c) {
+      if (fill_data->mask[j]) {
+        *c = fill_data->value[j];
+      }
+    }
+  }
+}
+
+
 static void
 fill_polygon(
         unsigned char *data,
         int pixelsize,
         int rowstride,
-        unsigned char value[4], 
+        unsigned char value[4],
         unsigned char mask[4],
-        GdkPoint *pts, 
-        int num_pts, 
+        GdkPoint *pts,
+        int num_pts,
         int xoffset,
         int yoffset)
 {
    /* this assumes the data is in RGBA format, each channel takes 1 bytes (thus 4 bytes per pixel) */
 
-   int x;
-   int j;
-   unsigned char *c;
-   
-/* taken from gd.c in gd 1.8.3 */
-   
    int i;
-   int y;
-   int miny, maxy;
-   int x1, y1;
-   int x2, y2;
-   int ind1, ind2;
-   int ints;
-   int * polyInts = 0;
-   int polyAllocated = 0;
-   
-   debug_fn();
-   
+   GdkRegion *region;
+   GdkRectangle bbox;
+   GdkSpan *spans;
+   GdkSpanFillData fill_data;
+
    g_assert(xoffset >= 0);
    g_assert(yoffset >= 0);
-   
-   if (!num_pts) {
-	   return;
-   }
-   if (!polyAllocated) {
-	   polyInts = (int *) malloc(sizeof(int) * num_pts);
-	   polyAllocated = num_pts;
-   }		
 
-   miny = pts[0].y;
-   maxy = pts[0].y;
-   for (i=1; (i < num_pts); i++) {
-	   if (pts[i].y < miny) {
-		   miny = pts[i].y;
-	   }
-	   if (pts[i].y > maxy) {
-		   maxy = pts[i].y;
-	   }
-   }
-   /* Fix in 1.3: count a vertex only once */
-   for (y=miny; (y <= maxy); y++) {
-/*1.4		int interLast = 0; */
-/*		int dirLast = 0; */
-/*		int interFirst = 1; */
-	   ints = 0;
-	   for (i=0; (i < num_pts); i++) {
-		   if (!i) {
-			   ind1 = num_pts-1;
-			   ind2 = 0;
-		   } else {
-			   ind1 = i-1;
-			   ind2 = i;
-		   }
-		   y1 = pts[ind1].y;
-		   y2 = pts[ind2].y;
-		   if (y1 < y2) {
-			   x1 = pts[ind1].x;
-			   x2 = pts[ind2].x;
-		   } else if (y1 > y2) {
-			   y2 = pts[ind1].y;
-			   y1 = pts[ind2].y;
-			   x2 = pts[ind1].x;
-			   x1 = pts[ind2].x;
-		   } else {
-			   continue;
-		   }
-		   if ((y >= y1) && (y < y2)) {
-			   polyInts[ints++] = (y-y1) * (x2-x1) / (y2-y1) + x1;
-		   } else if ((y == maxy) && (y > y1) && (y <= y2)) {
-			   polyInts[ints++] = (y-y1) * (x2-x1) / (y2-y1) + x1;
-		   }
-	   }
-	   qsort(polyInts, ints, sizeof(int), cmp_int);
+   debug_fn();
 
-	   for (i=0; (i < (ints)); i+=2) {
-	      c = data + (y - yoffset) * rowstride + (polyInts[i] - xoffset) * pixelsize;
-	      for (x = polyInts[i]; x <= polyInts[i+1] ; x++)
-	      {
-	         for (j = 0; j < pixelsize; j++, c++)
-             {
-                 if (mask[j])
-                     *c = value[j];
-             }    
-	      }
-	      
-	   }
+   if (!num_pts)
+     return;
+
+   region  = gdk_region_polygon(pts, num_pts, GDK_EVEN_ODD_RULE);
+
+   fill_data.rowstride = rowstride;
+   fill_data.pixelsize = pixelsize;
+   fill_data.data = data;
+   for (i = 0; i < 4; ++i) {
+     fill_data.value[i] = value[i];
+     fill_data.mask[i] = mask[i];
    }
 
-   if (polyInts)
-   {   
-       debug("freeing polyInts");
-       free(polyInts);
+   gdk_region_offset(region, -xoffset, -yoffset);
+
+   gdk_region_get_clipbox(region, &bbox);
+
+   spans = (GdkSpan *) g_new0(GdkSpan, bbox.height);
+   for(i = 0; i < bbox.height; ++i) {
+     spans[i].x = bbox.x;
+     spans[i].width = bbox.width;
+     spans[i].y = bbox.y + i;
    }
+
+   gdk_region_spans_intersect_foreach(region, spans, bbox.height, TRUE, gdk_span_fill, &fill_data);
+
+   g_free(spans);
+   gdk_region_destroy(region);
+
 }
 
 int
@@ -476,8 +495,8 @@ image_flip_x(gpaint_image *image)
 	        p4 -= pixel_size;
       }
     }
-    gdk_pixbuf_unref(image->pixbuf); 
-    image->pixbuf = newpixbuf;     
+    gdk_pixbuf_unref(image->pixbuf);
+    image->pixbuf = newpixbuf;
     return 0;
 }
 
@@ -488,10 +507,10 @@ image_flip_y(gpaint_image *image)
     GdkPixbuf *newpixbuf = gdk_pixbuf_new(
                             GDK_COLORSPACE_RGB,
                             TRUE,
-                            gdk_pixbuf_get_bits_per_sample(image->pixbuf), 
+                            gdk_pixbuf_get_bits_per_sample(image->pixbuf),
                             width = gdk_pixbuf_get_width(image->pixbuf),
                             height = gdk_pixbuf_get_height(image->pixbuf));
-   
+
     int rowstride = gdk_pixbuf_get_rowstride(newpixbuf);
     int x, y;
     int j;
@@ -515,15 +534,15 @@ image_flip_y(gpaint_image *image)
             }
             p3 += pixel_size;
             p4 += pixel_size;
-        } 
+        }
     }
-    gdk_pixbuf_unref(image->pixbuf); 
-    image->pixbuf = newpixbuf;     
+    gdk_pixbuf_unref(image->pixbuf);
+    image->pixbuf = newpixbuf;
     return 0;
 }
 
 
-/* the below 2d transform routine contains code 
+/* the below 2d transform routine contains code
    taken from xpaint, with the following copyright notice */
 /* +-------------------------------------------------------------------+ */
 /* | Copyright 1992, 1993, David Koblas (koblas@netcom.com)            | */
@@ -575,7 +594,7 @@ image_rotate(gpaint_image *image, double degrees)
     matrix[0][1] = -sin(radians);
     matrix[1][0] = sin(radians);
     matrix[1][1] = cos(radians);
-   
+
     /* find the new bounding box */
     width = gdk_pixbuf_get_width(image->pixbuf);
     height = gdk_pixbuf_get_height(image->pixbuf);
@@ -608,10 +627,10 @@ image_rotate(gpaint_image *image, double degrees)
     }
     width2 = maxx - minx;
     height2 = maxy - miny;
-   
-   
+
+
     INVERT_MAT(matrix, invmatrix);
-   
+
     p1 = gdk_pixbuf_get_pixels(image->pixbuf);
     newpixbuf = gdk_pixbuf_new(
            GDK_COLORSPACE_RGB,
@@ -623,9 +642,9 @@ image_rotate(gpaint_image *image, double degrees)
     rowstride2 = gdk_pixbuf_get_rowstride(newpixbuf);
     pixel_size = gdk_pixbuf_get_bits_per_sample(image->pixbuf) *
                   gdk_pixbuf_get_n_channels(image->pixbuf) / 8;
-   
+
 #define PIXEL(x, y) (p1 + ROUND(y) * rowstride1 + ROUND(x) * pixel_size)
-   
+
     for (y = miny; y < maxy; y++, p2 += rowstride2)
     {
         p4 = p2;
@@ -636,12 +655,12 @@ image_rotate(gpaint_image *image, double degrees)
             if (ny<0) ny = 0.0;
 	        nx += width / 2.0;
 	        ny += height / 2.0;
-   
+
             g_return_val_if_fail(nx >= 0, 1);
             g_return_val_if_fail(ny >= 0, 1);
             g_return_val_if_fail(nx <= gdk_pixbuf_get_width(image->pixbuf), 1);
             g_return_val_if_fail(ny <= gdk_pixbuf_get_height(image->pixbuf), 1);
-            
+
             p3 = PIXEL(nx, ny);
 	        for (j = 0; j < pixel_size; j++)
             {
@@ -649,9 +668,33 @@ image_rotate(gpaint_image *image, double degrees)
             }
         }
     }
-    gdk_pixbuf_unref(image->pixbuf);      
+    gdk_pixbuf_unref(image->pixbuf);
     image->pixbuf = newpixbuf;
     return 0;
 }
 
+int
+image_rotate_simple (gpaint_image *image, int degrees)
+{
+	GdkPixbuf *newpixbuf;
 
+	switch (degrees)
+	{
+        case GDK_PIXBUF_ROTATE_NONE:
+        case GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE:
+        case GDK_PIXBUF_ROTATE_UPSIDEDOWN:
+        case GDK_PIXBUF_ROTATE_CLOCKWISE:
+            newpixbuf = gdk_pixbuf_rotate_simple (image->pixbuf, degrees);
+            gdk_pixbuf_unref (image->pixbuf);
+            image->pixbuf = newpixbuf;
+            return 0;
+            break;
+        default:
+            return 1;
+    }
+}
+
+
+GdkPixbuf* image_pixbuf(gpaint_image* image) {
+    return image->pixbuf;
+};
