@@ -26,12 +26,14 @@
 #  include <config.h>
 #endif
 
+#include <stdlib.h>
+#include <string.h>
+
 #include "debug.h"
 #include "canvas.h"
 #include "image.h"
 #include "image_processing.h"
 #include "tool_palette.h"
-#include "menu.h"
 #include "print.h"
 #include "file.h"
 #include "text.h"
@@ -484,9 +486,9 @@ on_rotate_menu_activate                (GtkMenuItem     *menuitem,
 
     sscanf(name, "rotate_%c%d_menu", &sign, &degrees);
     debug2("sign = %c  degrees = %d", sign, degrees);
-    if (sign=='n')
+    if (sign=='p')
     {
-        degrees *= -1;
+        degrees = 360 - degrees;
     }
     canvas_focus_lost(canvas);
     drawing_rotate(canvas->drawing, degrees);
