@@ -1,6 +1,6 @@
 /* $Id: text.c,v 1.4 2005/01/27 02:53:01 meffie Exp $
  *
- * GNU Paint 
+ * GNU Paint
  * Copyright 2000-2003, 2007  Li-Cheng (Andy) Tai
  *
  * Authors: Li-Cheng (Andy) Tai
@@ -26,6 +26,8 @@
 #  include <config.h>
 #endif
 
+#include <ctype.h>
+#include <stdlib.h>
 #include <string.h>
 
 #ifdef HAVE_STRING_H
@@ -47,13 +49,14 @@
 typedef struct _gpaint_text
 {
     gpaint_tool tool;
-    
+
     gpaint_point location;
     gint timer;
     gint flash_state;
     GString *textbuf;
     int max_width;
     int max_height;
+    GtkFontButton *fontpicker;
 } gpaint_text;
 
 
@@ -75,6 +78,7 @@ static void text_draw_cursor(gpaint_text *text);
 static void text_draw_string(gpaint_text *text);
 static gint text_handle_timeout(gpaint_text *text);
 static void text_draw_current_string(gpaint_tool *tool);
+extern void convert_color(const GdkColor *color, unsigned char *r, unsigned char *g, unsigned char *b);
 
 gpaint_tool *text_create(const char *name)
 {
@@ -92,12 +96,13 @@ gpaint_tool *text_create(const char *name)
     GPAINT_TOOL(text)->change         = text_focus_change;
     GPAINT_TOOL(text)->current_draw   = text_draw_current_string;
     GPAINT_TOOL(text)->commit_change  = text_commit_change;
-    
+
     text->textbuf =   g_string_new(0);
+    text->fontpicker = NULL;
     return GPAINT_TOOL(text);
 }
 
-static void 
+static void
 text_destroy(gpaint_tool *tool)
 {
     gpaint_text *text = GPAINT_TEXT(tool);
@@ -106,7 +111,7 @@ text_destroy(gpaint_tool *tool)
     g_string_free(text->textbuf, TRUE);
     if (text->timer)
         gtk_timeout_remove(text->timer);
-  
+
     g_free(tool);
 }
 
@@ -118,6 +123,7 @@ text_select(gpaint_tool *tool)
     g_string_printf(text->textbuf, "");
     text->timer = g_timeout_add(TEXT_CURSOR_BLINK_RATE,
                      (GtkFunction)(text_handle_timeout), text);
+    gtk_widget_set_sensitive(GTK_WIDGET(text->fontpicker), TRUE);
 }
 
 static void
@@ -126,7 +132,7 @@ text_clear(gpaint_text *text)
     clear_point(&(text->location));
     text->max_width = text->max_height = 0;
     g_string_printf(text->textbuf, "");
-  
+
 }
 
 static void
@@ -138,14 +144,14 @@ text_deselect(gpaint_tool *tool)
     g_assert(text->timer);
     gtk_timeout_remove(text->timer);
     text->timer = 0;
-    
+
     text_clear_cursor(text);
     if (is_point_defined(&text->location) && g_utf8_strlen(text->textbuf->str, -1))
     {
         text_draw_string(text);
     }
     text_clear(text);
-   
+    gtk_widget_set_sensitive(GTK_WIDGET(text->fontpicker), FALSE);
 }
 
 static gboolean
@@ -153,15 +159,18 @@ text_attribute(gpaint_tool* tool, gpaint_attribute attrib, gpointer data)
 {
     debug_fn();
     GtkStyle *style;
-    GtkWidget *widget = GTK_WIDGET(tool->canvas->drawing_area);
+    GtkWidget *widget = NULL;
+
+    if ((tool->canvas == NULL) || (tool->canvas->drawing_area == NULL)) return FALSE;
+    widget = GTK_WIDGET(tool->canvas->drawing_area);
     style = gtk_widget_get_style(widget);
     g_assert(style);
-      
+
     if (attrib == GpaintFont)
     {
         gpaint_text *text = GPAINT_TEXT(tool);
         gtk_widget_modify_font(widget, pango_font_description_from_string((char*) data));
-      
+
         return TRUE;
     }
     return FALSE;
@@ -175,7 +184,7 @@ text_focus_change(gpaint_tool *tool, gpaint_change change, gpointer data)
     {
         text_clear_cursor(GPAINT_TEXT(tool));
     }
-   
+
 }
 
 static void
@@ -189,7 +198,7 @@ text_button_press(gpaint_tool *tool, int x, int y)
             text_draw_string(text);
         }
         text_clear_cursor(text);
-     
+
         text_clear(text);
     }
 }
@@ -218,7 +227,7 @@ text_key_release(gpaint_tool *tool, GdkEventKey *keyevent)
     int length;
     char *tmp = 0;
     int width, height;
-   
+
     text_clear_cursor(text);
     gdk_gc_get_values(drawing->gc, &gcvalues);
     tmp = strdup(text->textbuf->str);
@@ -232,7 +241,9 @@ text_key_release(gpaint_tool *tool, GdkEventKey *keyevent)
     }
     else if (keyevent->string)
     {
+      if (isprint(keyevent->string[0])) {
         g_string_append(text->textbuf, keyevent->string);
+      }
     }
     else if ((keyevent->keyval >= GDK_space) && (keyevent->keyval < GDK_Shift_L))
     {
@@ -243,42 +254,42 @@ text_key_release(gpaint_tool *tool, GdkEventKey *keyevent)
     {
         g_string_printf(text->textbuf, "");
     }
-   
+
     layout = gtk_widget_create_pango_layout(GTK_WIDGET(tool->canvas->drawing_area), tmp);
     style = gtk_widget_get_style(GTK_WIDGET(tool->canvas->drawing_area));
-    
+
     pango_layout_get_pixel_size(layout, &width, &height);
     if (text->max_width < width) text->max_width = width;
     if (text->max_height < height) text->max_height = height;
-    
-    
+
+
     if (tmp && g_utf8_strlen(tmp, -1))
     {
-        gdk_draw_pixmap(drawing->window, drawing->gc, drawing->backing_pixmap, 
+        gdk_draw_pixmap(drawing->window, drawing->gc, drawing->backing_pixmap,
                          text->location.x, text->location.y,
                          text->location.x, text->location.y,
                          text->max_width, text->max_height);
     }
-    
+
     pango_layout_set_text(layout, text->textbuf->str, -1);
     pango_layout_get_pixel_size(layout, &width, &height);
     if (text->max_width < width) text->max_width = width;
     if (text->max_height < height) text->max_height = height;
     {
-        gdk_draw_pixmap(drawing->window, drawing->gc, drawing->backing_pixmap, 
+        gdk_draw_pixmap(drawing->window, drawing->gc, drawing->backing_pixmap,
                          text->location.x, text->location.y,
                          text->location.x, text->location.y,
                          text->max_width, text->max_height);
         set_layout_foreground_color(layout, &(gcvalues.foreground));
         gtk_paint_layout(style, drawing->window,
-                        GTK_STATE_NORMAL, TRUE, NULL, 
-                        GTK_WIDGET(tool->canvas->drawing_area), NULL, 
+                        GTK_STATE_NORMAL, TRUE, NULL,
+                        GTK_WIDGET(tool->canvas->drawing_area), NULL,
                         text->location.x, text->location.y,
                         layout);
-       
+
     }
-   
-   
+
+
     if (keyevent->keyval == GDK_Return)
     {
         g_string_append(text->textbuf, "\n");
@@ -299,24 +310,24 @@ static void text_draw_current_string(gpaint_tool *tool)
     GdkGCValues gcvalues;
     if (text && is_point_defined(&text->location))
     {
-        gpaint_drawing *drawing = tool->drawing;    
+        gpaint_drawing *drawing = tool->drawing;
         gdk_gc_get_values(drawing->gc, &gcvalues);
-    
+
         PangoLayout *layout = gtk_widget_create_pango_layout(GTK_WIDGET(tool->canvas->drawing_area), text->textbuf->str);
-      
+
         GtkStyle *  style = gtk_widget_get_style(GTK_WIDGET(tool->canvas->drawing_area));
-        
+
         set_layout_foreground_color(layout, &(gcvalues.foreground));
         gtk_paint_layout(style, drawing->window,
-                        GTK_STATE_NORMAL, TRUE, NULL, 
-                        GTK_WIDGET(tool->canvas->drawing_area), NULL, 
+                        GTK_STATE_NORMAL, TRUE, NULL,
+                        GTK_WIDGET(tool->canvas->drawing_area), NULL,
                         text->location.x, text->location.y,
                         layout);
-       
+
         g_object_unref(layout);
-        
+
     }
-    
+
 
 }
 
@@ -327,14 +338,14 @@ text_draw_string(gpaint_text *text) /* draw string permanently */
     gpaint_drawing *drawing = tool->drawing;
     GdkGCValues gcvalues;
     PangoLayout *layout = gtk_widget_create_pango_layout(GTK_WIDGET(tool->canvas->drawing_area), text->textbuf->str);
-  
+
     GtkStyle *  style = gtk_widget_get_style(GTK_WIDGET(tool->canvas->drawing_area));
     gdk_gc_get_values(drawing->gc, &gcvalues);
-   
+
     set_layout_foreground_color(layout, &(gcvalues.foreground));
     gtk_paint_layout(style, drawing->backing_pixmap,
-                    GTK_STATE_NORMAL, TRUE, NULL, 
-                    GTK_WIDGET(tool->canvas->drawing_area), NULL, 
+                    GTK_STATE_NORMAL, TRUE, NULL,
+                    GTK_WIDGET(tool->canvas->drawing_area), NULL,
                     text->location.x, text->location.y,
                     layout);
     drawing_modified(drawing);
@@ -360,14 +371,14 @@ text_handle_timeout(gpaint_text *text)
     if (text == 0)
         return 0;
     if (GPAINT_TOOL(text)->canvas->has_focus)
-    {    
+    {
         //debug("tick");
         text_draw_cursor(text);
     }
     else
-    {    
+    {
         text_clear_cursor(text);
-    }   
+    }
     return 1;
 }
 
@@ -394,56 +405,56 @@ text_draw_cursor(gpaint_text *text)
     PangoLayout *layout = 0;
     PangoRectangle rect, rect2;
     GtkStyle *  style = gtk_widget_get_style(GTK_WIDGET(tool->canvas->drawing_area));
-  
+
     gdk_gc_get_values(drawing->gc, &gcvalues);
     layout = gtk_widget_create_pango_layout(GTK_WIDGET(tool->canvas->drawing_area), text->textbuf->str);
-    
+
     if (is_point_defined(&text->location))
     {
         pango_layout_get_pixel_size(layout, &(rect.width), &(rect.height));
-        
+
         if (text->max_width < rect.width) text->max_width = rect.width;
         if (text->max_height < rect.height) text->max_height = rect.height;
-        
+
         pango_layout_index_to_pos(layout, g_utf8_strlen(text->textbuf->str, -1), &rect);
         rect.x = PANGO_PIXELS(rect.x);
         rect.y = PANGO_PIXELS(rect.y);
         rect.width = PANGO_PIXELS(rect.width);
         rect.height = PANGO_PIXELS(rect.height);
-        
-        gdk_draw_pixmap(drawing->window, drawing->gc, drawing->backing_pixmap, 
+
+        gdk_draw_pixmap(drawing->window, drawing->gc, drawing->backing_pixmap,
                          text->location.x + rect.x , text->location.y + rect.y,
                          text->location.x + rect.x , text->location.y + rect.y,
                          TEXT_CURSOR_WIDTH + 2, rect.height + 2);
-	        
+
 
         gdk_color_white(gdk_rgb_get_cmap(), &white);
-        gdk_gc_set_foreground(drawing->gc, &white); 
+        gdk_gc_set_foreground(drawing->gc, &white);
 
         gdk_gc_set_function(drawing->gc, GDK_INVERT);
         gdk_gc_set_line_attributes(drawing->gc,
-                                   TEXT_CURSOR_WIDTH / 2, 
+                                   TEXT_CURSOR_WIDTH / 2,
                                    GDK_LINE_SOLID,
                                    GDK_CAP_PROJECTING,
                                    GDK_JOIN_ROUND);
-    
-        if (text->flash_state != 0)                 
+
+        if (text->flash_state != 0)
             gdk_draw_line(drawing->window,
                           drawing->gc,
                           text->location.x + rect.x + TEXT_CURSOR_WIDTH / 2,
                           text->location.y + rect.y,
                           text->location.x + rect.x + TEXT_CURSOR_WIDTH / 2,
-                          text->location.y + rect.y + rect.height - 1); 
+                          text->location.y + rect.y + rect.height - 1);
 
-        gdk_gc_set_function(drawing->gc, gcvalues.function);   
-        gdk_gc_set_line_attributes(drawing->gc, 
+        gdk_gc_set_function(drawing->gc, gcvalues.function);
+        gdk_gc_set_line_attributes(drawing->gc,
                                    gcvalues.line_width,
                                    gcvalues.line_style,
-                                   gcvalues.cap_style, 
+                                   gcvalues.cap_style,
                                    gcvalues.join_style);
         gdk_gc_set_foreground(drawing->gc, &(gcvalues.foreground));
-        text->flash_state = !(text->flash_state); 
-       
+        text->flash_state = !(text->flash_state);
+
     }
     g_object_unref(layout);
 }
@@ -459,8 +470,8 @@ static void set_layout_foreground_color(PangoLayout *layout, GdkColor *foregroun
         return;
     convert_color(foreground, &red, &green, &blue);
     color_attr = (PangoAttrColor*) pango_attr_foreground_new(color16(red), color16(green), color16(blue));
-    
-    
+
+
     if (attrs == 0)
         attrs = pango_attr_list_new();
     else
@@ -469,11 +480,13 @@ static void set_layout_foreground_color(PangoLayout *layout, GdkColor *foregroun
     color_attr->attr.end_index = g_utf8_strlen(text, -1);
     pango_attr_list_change(attrs, (PangoAttribute *) color_attr);
     pango_layout_set_attributes(layout, attrs);
-    
+
     pango_attr_list_unref(attrs);
-  
+
 }
 
-   
-
-
+void text_set_fontpicker(gpaint_tool *tool, GtkFontButton *fontpicker)
+{
+    gpaint_text *text = GPAINT_TEXT(tool);
+    text->fontpicker = fontpicker;
+}
